#!/bin/bash

# Script mk-atalaia-certificates.sh
# Version 202311211914

# Variables

# Curva ECDSA
ECDSA_CURVE="secp256k1"

# Tamaño de clave RSA en número de bits
RSA_KEY_SIZE="3072"

# Validez del certificado en número de días
VALIDITY_DAYS="365"

# Sufijo DNS
DNS_SUFFIX="atalaia.local"

# Parte común del Nombre Distinguido (DN) del sujeto para todos los certificados
CERTS_SUBJECT="/C=ES/ST=A Coruña/L=Santiago de Compostela/O=Atalaia/OU=ATIC" 

# Nombre común (CN) de certificado raíz
ROOT_CA="atalaia-root-ca"

# Ruta en donde se crean los certificados y las claves privadas asociadas
CERTIFICATES_PATH="$(pwd)/backups/certs"

# Ruta con el certificado y clave privada del certificado raíz
ROOT_CERTIFICATES_PATH="$(pwd)/backups/root-certs"

# Ruta con los certificados y claves privadas necesarios para Wazuh
WZH_PATH="$(pwd)/config/certs"

# Nombres comunes (CN) de componentes centrales de la plataforma Wazuh
WZH_HOST=("admin" "indexer" "manager" "dashboard")

# Funciones

# Función de ayuda
function help_command () {
    echo
    echo "Modo de empleo:"
    echo " $(basename $0) {rsa|ecdsa} root_ca"
    echo " $(basename $0) {rsa|ecdsa} file"
    echo " $(basename $0) help"            
    echo
    echo "Claves criptográficas:"
    echo " rsa: Usar claves RSA para generar los certificados."
    echo " ecdsa: Usar claves ECDSA para generar los certificados."
    echo
    echo "Fichero de entrada:"
    echo " file: Crear certificados para los hosts a partir del archivo indicado."
    echo "       Debe estar ubicado en la ruta del script $(basename $0)."
    echo "       Cada línea tiene una asignación de nombre de host y de dirección IP"
    echo "       separados por espacios o tabuladores."
    echo
    echo "Certificado raíz de CA:"
    echo " root_ca: Crear certificado para la autoridad de certificación raíz."
    echo
    echo "Ayuda:"
    echo " help: Solicitar ayuda."
    echo
}

# Crear los directorios necesarios
function create_dirs () {
    if [ "${ROOT_CA_PARAMETER}" == "root_ca" ]; then
        if [ ! -d "${ROOT_CERTIFICATES_PATH}" ]; then
            echo -n "El directorio con el certificado raíz y su clave privada no existe."
            echo " Creándolo..."
            echo "Ruta: ${ROOT_CERTIFICATES_PATH}"
            mkdir -p ${ROOT_CERTIFICATES_PATH}
        else
            echo "El directorio ${ROOT_CERTIFICATES_PATH} existe."
            echo "Creando backup en ${ROOT_CERTIFICATES_PATH}-$(date +%Y%m%d%H%M%S)..."
            mv ${ROOT_CERTIFICATES_PATH} ${ROOT_CERTIFICATES_PATH}-$(date +%Y%m%d%H%M%S)
            mkdir -p ${ROOT_CERTIFICATES_PATH}
        fi
    elif [ -z ${ROOT_CA_PARAMETER} ]; then
        if [ ! -d "${CERTIFICATES_PATH}" ]; then
            echo "Creando directorio ${CERTIFICATES_PATH} para generar certificados..."
            mkdir -p ${CERTIFICATES_PATH}
        else
            echo -n "El directorio ${CERTIFICATES_PATH} existe."
            echo " Creando backup en ${CERTIFICATES_PATH}-$(date +%Y%m%d%H%M%S)"
            mv "${CERTIFICATES_PATH}" "${CERTIFICATES_PATH}-$(date +%Y%m%d%H%M%S)"
            mkdir -p ${CERTIFICATES_PATH}
        fi
    fi
   
    if [ ! -d "${WZH_PATH}" ]; then
      echo -n "El directorio con certificados para la plataforma de seguridad Wazuh"
      echo " no existe. Creándolo..."
      echo "Ruta: ${WZH_PATH}"
      mkdir -p ${WZH_PATH}
    fi    
}

# ROOT CA
function create_root_ca () {
    echo "Creando certificado raíz ${CERTS_SUBJECT}/CN=${ROOT_CA}.${DNS_SUFFIX}..."
    # Clave privada
    if [ ${SELECTED_ALGORITHM} == "ecdsa" ]; then
    openssl ecparam -name ${ECDSA_CURVE} -genkey -noout \
    -out ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.key
    elif [ ${SELECTED_ALGORITHM} == "rsa" ];then
    openssl genrsa -out ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.key ${RSA_KEY_SIZE} 
    fi
    # Certificado
    openssl req -x509 -sha512 -utf8 -noenc -days ${VALIDITY_DAYS} \
    -subj "${CERTS_SUBJECT}/CN=${ROOT_CA}.${DNS_SUFFIX}" \
    -key ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.key \
    -out ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt 2> /dev/null
    # Permisos
    chmod 400 ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.{crt,key}
}

# Certificados
function create_certificates () {
    # Acceso a los arrays WZH_HOST y WZH_IP mediante índices
    while read endpointName endpointIP;
    #for CERTIFICATE_INDEX in ${!WZH_HOST[@]}
    do
        CERT_OPTIONS="[CERT_CONFIG]\n\
        subjectAltName = DNS:${endpointName}.${DNS_SUFFIX},\
        IP:${endpointIP}\n\
        basicConstraints = CA:FALSE\n\
        keyUsage = nonRepudiation, digitalSignature, keyEncipherment, dataEncipherment"
          
        echo -n "Creando certificado ${CERTS_SUBJECT}"
        echo "/CN=${endpointName}.${DNS_SUFFIX}..."
        # Clave privada
        if [ ${SELECTED_ALGORITHM} == "ecdsa" ]; then
        openssl ecparam -name ${ECDSA_CURVE} -genkey -noout \
        -out ${CERTIFICATES_PATH}/${endpointName}-temp.key    
        elif [ ${SELECTED_ALGORITHM} == "rsa" ];then
        openssl genrsa \
        -out ${CERTIFICATES_PATH}/${endpointName}-temp.key ${RSA_KEY_SIZE}
        fi
        # Clave privada PKCS#8 
        openssl pkcs8 -inform PEM -outform PEM \
        -in ${CERTIFICATES_PATH}/${endpointName}-temp.key \
        -topk8 -nocrypt -v1 PBE-SHA1-3DES \
        -out ${CERTIFICATES_PATH}/${endpointName}.key
        # CSR
        openssl req -new \
        -subj "${CERTS_SUBJECT}/CN=${endpointName}.${DNS_SUFFIX}" \
        -reqexts CERT_CONFIG -config <(echo -e "${CERT_OPTIONS}") \
        -utf8 -key ${CERTIFICATES_PATH}/${endpointName}.key \
        -out ${CERTIFICATES_PATH}/${endpointName}.csr
        # Certificado
        openssl x509 -req -in ${CERTIFICATES_PATH}/${endpointName}.csr \
        -CA ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt \
        -CAkey ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.key -copy_extensions copy \
        -CAcreateserial -sha512 -days 365 \
        -out ${CERTIFICATES_PATH}/${endpointName}.crt 2> /dev/null
    done < ${HOSTS_FILE}
}

# Mostar datos del certificado raíz de la Autoridad de Certificción (CA)
function show_root_CA_certificate () {
    echo -e "\n---- Certificado ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt----"
    openssl x509 -in ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt -noout -subject \
    -ext basicConstraints,subjectAltName,keyUsage -dates -nameopt oneline,-esc_msb
}

# Mostar datos de los certificados
function show_certificates () {
    while read endpointName endpointIP;
    do
        echo -e "\n---- Certificado ${endpointName}.crt----"
        openssl x509 -in ${CERTIFICATES_PATH}/${endpointName}.crt -noout -subject \
        -ext basicConstraints,subjectAltName,keyUsage -dates -nameopt oneline,-esc_msb
    done < ${HOSTS_FILE}
    echo "----"
}

# Copiar certificados y claves privadas a ${WZH_PATH} para despliegue en Docker Compose
# No se copia clave privada del certificado ROOT CA
function copy_certificates () {
    if [ "${ROOT_CA_PARAMETER}" == "root_ca" ] && 
    [ -f ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt ]; then
        echo "Copiando certificado raíz de la autoridad de certificación (CA)..."
        echo -e "\tClave privada: ${ROOT_CA}.crt"
        echo -e "\tDe: ${ROOT_CERTIFICATES_PATH}/"
        echo -e "\tA: ${WZH_PATH}/"
        cp -f ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt ${WZH_PATH}/
        chmod 400 ${WZH_PATH}/${ROOT_CA}.crt
    elif [ -z "${ROOT_CA_PARAMETER}" ]; then
        # Certificados y claves privadas de los hosts
        for CERTIFICATE in ${WZH_HOST[@]}
        do
            if [ -f ${CERTIFICATES_PATH}/${CERTIFICATE}.crt ]; then
                echo "Copiando certificado ${CERTIFICATE}.crt:"
                echo -e "\tDe: ${CERTIFICATES_PATH}/"
                echo -e "\tA: ${WZH_PATH}/"
                cp -f ${CERTIFICATES_PATH}/${CERTIFICATE}.crt ${WZH_PATH}/
                chmod 400 ${WZH_PATH}/${CERTIFICATE}.crt
            fi
            if [ -f ${CERTIFICATES_PATH}/${CERTIFICATE}.key ]; then
                echo "Copiando clave privada ${CERTIFICATE}.key:"
                echo -e "\tDe: ${CERTIFICATES_PATH}/"
                echo -e "\tA: ${WZH_PATH}/"
                cp -f ${CERTIFICATES_PATH}/${CERTIFICATE}.key ${WZH_PATH}
                chmod 400 ${WZH_PATH}/${CERTIFICATE}.key
            fi
        done    
    fi
    # Permisos
    # chmod 400 ${WZH_PATH}/*
}

# Ejecución del script

umask 7077

# Condiciones de ejecución
if [ $# != 2  ]; then
    help_command
else
    SELECTED_ALGORITHM="$1"  
    case "${SELECTED_ALGORITHM}" in
        rsa|ecdsa) 
            if [ $2 != "root_ca" ] && [ ! -f $(pwd)/$2 ]; then
                echo 
                echo "ERROR: No existe el fichero de hosts indicado: $(pwd)/$2"
                echo
            else
                echo -en "\nUso de claves "
                echo "$(echo "${SELECTED_ALGORITHM}" | tr [:lower:] [:upper:])"
                echo
                if [ $2 == "root_ca" ]; then
                    ROOT_CA_PARAMETER=$2
                    create_dirs
                    create_root_ca
                    copy_certificates
                    show_root_CA_certificate
                elif [ -f $(pwd)/$2 ]; then
                    if [ -f ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.crt ] || 
                    [ -f ${ROOT_CERTIFICATES_PATH}/${ROOT_CA}.key ] ; then
                        HOSTS_FILE="$2"
                        create_dirs
                        create_certificates
                        copy_certificates
                        show_certificates
                    else
                        echo -n "ERROR: No existen el certificados raíz y/o la "
                        echo "clave privada de la autoridad de certificación."
                        echo -n "Se pueden generar con el siguiente comando: "
                        echo "$(basename $0) {rsa|ecdsa} root_ca"
                        echo
                    fi
                fi
            fi
            ;;
        *) help_command;;
    esac
fi
